#ifndef __LFS_PTRACE_H__
#define __LFS_PTRACE_H__

#include <sys/ptrace.h>
#include <sys/wait.h>
#include <asm/ptrace.h>
#include <sys/time.h>
#include <asm/unistd.h>

#include <stdlib.h>

#define LOG_VEC_ENTRIES		4
#define HASH_SIZE		16

struct event {
	int orig_eax;
	struct timeval tv0, tv1;
};

struct log_vec {
	struct event events[LOG_VEC_ENTRIES];
	int last;
	struct log_vec *next;
};

struct ptraced_child {
	pid_t pid;
	struct event cur_event;
	struct log_vec *log, *cur;
	struct ptraced_child *next;
};

#define __NR_syscall_max	300

struct syscall_name {
	const char *name;
	int track;
};

extern struct syscall_name syscalls[__NR_syscall_max];

#define OFFSET ((unsigned long)&((struct pt_regs *)0)->orig_rax)
#define HASH(v) (v % HASH_SIZE)

#define err(msg...) do {						\
	fprintf(stderr, "%s:%d ", __FILE__, __LINE__);			\
	fprintf(stderr, msg);						\
	exit(-1);							\
} while (0)

#define xcalloc(ptr, size) do {						\
	ptr = calloc(1, (size));					\
	if (!ptr)							\
		err("not enough memory\n");				\
} while (0)

#define do_for_each_ptrace_event(ev)						\
	do {								\
		struct ptraced_child *pc;				\
		struct log_vec *vec;					\
		int i, j;						\
		for (i = 0; i < HASH_SIZE; i++) {			\
			for (pc = ptraced_table[i]; pc; pc = pc->next) { \
				for (vec = pc->log; vec; vec = vec->next) { \
					for (j = 0; j < vec->last; j++) { \
						ev = &vec->events[j];

#define while_for_each_ptrace_event()						\
					}				\
				}					\
			}						\
		}							\
	} while (0)

extern struct ptraced_child *ptraced_table[HASH_SIZE];

static inline double time_to_double(struct timeval *tv)
{
	return tv->tv_sec + tv->tv_usec / 1000000.0;
}

typedef void transf_cb_t(double val, double f, double *re, double *im);

extern struct timeval tracing_start;

void ptrace_get_events(unsigned long obs_time_us);
void ptrace_call_on_events(transf_cb_t cb, double f, double *re, double *im);
struct ptraced_child *find_child(pid_t pid);
void log_event(struct ptraced_child *pc, struct event *event);

#endif /* __LFS_PTRACE_H__ */
